/*
 * Ben NanoNote board late initialisation, based on uboot-xburst and xburst-tools.
 *
 * Copyright (C) 2000-2009 Wolfgang Denk, DENX Software Engineering, <wd@denx.de>
 * Copyright (C) 2006 Ingenic Semiconductor, <jlwei@ingenic.cn>
 * Copyright (C) Xiangfu Liu <xiangfu.z@gmail.com>
 * Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board.h"
#include "nanonote.h"

/* Later initialisation functions. */

void gpio_init2(void)
{
	/*
	 * Initialize LCD pins
	 */
	__gpio_as_slcd_8bit();

	/*
	 * Initialize MSC pins
	 */
	__gpio_as_msc();

	/*
	 * Initialize Other pins
	 */
	unsigned int i;
	for (i = 0; i < 7; i++){
		__gpio_as_input(GPIO_KEYIN_BASE + i);
		__gpio_enable_pull(GPIO_KEYIN_BASE + i);
	}

	for (i = 0; i < 8; i++) {
		__gpio_as_output(GPIO_KEYOUT_BASE + i);
		__gpio_clear_pin(GPIO_KEYOUT_BASE + i);
	}

	/* enable the TP4, TP5 as UART0 */
	__gpio_jtag_to_uart0();

	__gpio_as_input(GPIO_KEYIN_8);
	__gpio_enable_pull(GPIO_KEYIN_8);

	__gpio_as_output(GPIO_AUDIO_POP);
	__gpio_set_pin(GPIO_AUDIO_POP);

	__gpio_as_output(GPIO_LCD_CS);
	__gpio_clear_pin(GPIO_LCD_CS);

	__gpio_as_output(GPIO_AMP_EN);
	__gpio_clear_pin(GPIO_AMP_EN);

	__gpio_as_output(GPIO_SDPW_EN);
	__gpio_disable_pull(GPIO_SDPW_EN);
	__gpio_clear_pin(GPIO_SDPW_EN);

	__gpio_as_input(GPIO_SD_DETECT);
	__gpio_disable_pull(GPIO_SD_DETECT);

	__gpio_as_input(GPIO_USB_DETECT);
	__gpio_enable_pull(GPIO_USB_DETECT);
}

void cpm_init(void)
{
	__cpm_stop_ipu();
	__cpm_stop_cim();
	__cpm_stop_i2c();
	__cpm_stop_ssi();
	__cpm_stop_uart1();
	__cpm_stop_sadc();
	__cpm_stop_uhc();
	__cpm_stop_udc();
	__cpm_stop_aic1();
/*	__cpm_stop_aic2();*/
}

void rtc_init(void)
{
	while ( !__rtc_write_ready());
	__rtc_enable_alarm();	/* enable alarm */

	while ( !__rtc_write_ready());
	REG_RTC_RGR   = 0x00007fff; /* type value */

	while ( !__rtc_write_ready());
	REG_RTC_HWFCR = 0x0000ffe0; /* Power on delay 2s */

	while ( !__rtc_write_ready());
	REG_RTC_HRCR  = 0x00000fe0; /* reset delay 125ms */
}

/* Timer routines. */

unsigned long timestamp;
unsigned long lastdec;

/*
 * timer without interrupts
 */

void timer_init(void)
{
	__tcu_select_extalclk(TIMER_CHAN);
	__tcu_select_clk_div256(TIMER_CHAN);
	__tcu_set_count(TIMER_CHAN, 0);
	__tcu_set_half_data(TIMER_CHAN, 0);
	__tcu_set_full_data(TIMER_CHAN, TIMER_FDATA);

	__tcu_mask_half_match_irq(TIMER_CHAN);
	__tcu_mask_full_match_irq(TIMER_CHAN);
	__tcu_start_timer_clock(TIMER_CHAN);
	__tcu_start_counter(TIMER_CHAN);

	lastdec = 0;
	timestamp = 0;
}
